/*
  This file is part of Solver.

    Solver is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Solver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Solver. If not, see <http://www.gnu.org/licenses/>.
*/

package infra;

import infra.solution.ProblemSolution;
import infra.solver.SolverConfig;

public class Runner {
	
	public static String readParam(String[] args, String param, String defaultValue) {
		for (int i = 0; i < args.length - 1; i++) {
			if (args[i].equalsIgnoreCase("--" + param)) {
				return args[i + 1];
			}
		}
		return defaultValue;
	}
	
	public static void info() {
		System.out.println();
		System.out.println("Wywolanie:");
		System.out.println("java -jar Solver.jar [--in definition.xml] [--out out.xml] [--fixin plan.xml --fixdate yyyy.MM.dd] [--iters 100] [--itertime 1]");
	}
	
	public static void main(String[] args) {
		System.out.println("Infra S.A. Solver, ver. 0.4.20181030");
		info();
		String fixSolutionFile = readParam(args, "fixin", null);
		String fixDateString = readParam(args, "fixdate", null);
		if ((fixSolutionFile != null) && (fixDateString == null)) {
			System.err.println("Parametr 'fixin' wymaga podania parametru 'fixdate'");
			return;
		}
		
		String problemFile = readParam(args, "in", "definition.xml");
		String outputFile = readParam(args, "out", (fixSolutionFile == null) ? "out.xml" : "out_fix.xml");
		String searchItersStr = readParam(args, "iters", "100");
		String searchIterTimeStr = readParam(args, "itertime", "1");
		
		
		int searchIters = Integer.parseInt(searchItersStr);
		int searchIterTime = Integer.parseInt(searchIterTimeStr);
		System.out.println("Plik problemu        : " + problemFile);
		System.out.println("Iteracje wyszukiwania: " + searchIters);
		System.out.println("Czas iteracji [sek]  : " + searchIterTimeStr);
		
		SolverConfig config = new SolverConfig();
		config.searchIters = searchIters;
		config.searchTime = searchIterTime;
		
		Solver solver = new Solver(config);
		ProblemSolution solution = solver.solve(problemFile, fixSolutionFile, fixDateString);
		if (solution != null) {
			solution.serializeToXml(outputFile);
		}
	}
}
