/*
  This file is part of Solver.

    Solver is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Solver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Solver. If not, see <http://www.gnu.org/licenses/>.
*/

package infra;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

public class Utils {
	
	public static <T> List<T> flatten(List<List<T>> twoLevels) {
		List<T> result = new ArrayList<>();
		for (List<T> second : twoLevels) {
			result.addAll(second);
		}
		return result;
	}
	
	public static Calendar parse(String text) {
		if (text == null) {
			return null;
		}
		DateFormat df = new SimpleDateFormat("yyyy.MM.dd");
		Date date = null;
		try {
			date = df.parse(text);
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		return cal;
	}
	
	public static int parseInt(String text, int defValue) {
		try {
			int val = Integer.parseInt(text);
			return val;
		} catch (Exception e) {
			return defValue;
		}
	}
	
	public static String build(Calendar cal) {
		DateFormat df = new SimpleDateFormat("yyyy.MM.dd");
		Date date = cal.getTime();
		return df.format(date);
	}
}
