/*
  This file is part of Solver.

    Solver is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Solver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Solver. If not, see <http://www.gnu.org/licenses/>.
*/
package infra.definition;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import infra.solution.ProblemSolution;

public class ProblemDefinition {
	
	public enum WeeklyWorkload {
		CALENDAR,
		PERIOD,
		DAILY
	}
	
	private TimePeriod analysisPeriod;
	private Map<String, Watch> daysDefinition = new HashMap<String, Watch>();
	private List<Employee> employees = new ArrayList<Employee>();
	
	public WeeklyWorkload weeklyWorkload = WeeklyWorkload.CALENDAR;
	
	private int minsPerDay;
	private int minsPerWatch;
	private int minimumShortWatchMins;
	private int weekWatchLimit;
	
	public ProblemSolution existingSolution;
	public Calendar recomputationDate;
	
	public TimePeriod getAnalysisPeriod() {
		return analysisPeriod;
	}
	public void setAnalysisPeriod(TimePeriod analysisPeriod) {
		this.analysisPeriod = analysisPeriod;
	}
	public List<Employee> getEmployees() {
		return employees;
	}
	public Map<String, Watch> getDaysDefinition() {
		return daysDefinition;
	}
	
	public int getMinsPerWatch() {
		return minsPerWatch;
	}
	public void setMinsPerWatch(int minsPerWatch) {
		this.minsPerWatch = minsPerWatch;
	}
	
	public int getWeekWatchLimit() {
		return weekWatchLimit;
	}
	public void setWeekWatchLimit(int weekWatchLimit) {
		this.weekWatchLimit = weekWatchLimit;
	}
	
	public int getMinimumShortWatchMins() {
		return minimumShortWatchMins;
	}
	public void setMinimumShortWatchMins(int minimumShortWatchMins) {
		this.minimumShortWatchMins = minimumShortWatchMins;
	}
	
	public int employeeMinsPerPeriod(int i) {
		return analysisPeriod.getMinutesPerPeriod(employees.get(i));
		//Employee emp = employees.get(i);
		//int workDays = analysisPeriod.getWorkDaysPerPeriod(emp.getLeaveOfAbsence());
		//return (int)(emp.getMinsPerDay() * workDays);
	}
	
	public void print() {
		
		int totalMinsPerMonth = minsPerDay * analysisPeriod.getWorkDaysPerPeriod();
		//int totalWatchesPerMonth = totalMinsPerMonth / minsPerWatch;
		
		SimpleDateFormat fmt = new SimpleDateFormat("dd-MMM-yyyy");
		System.out.println("Data poczatku okresu         = " + fmt.format(analysisPeriod.startDate.getTime()));
		System.out.println("Data konca okresu            = " + fmt.format(analysisPeriod.endDate.getTime()));
		System.out.println("Dni wolne od pracy           :");
		for (int i = 0; i < analysisPeriod.getAllDaysPerPeriod(); i++) {
			if (analysisPeriod.getDayCode(i).equals("*")) {
				Calendar date = analysisPeriod.allDays.get(i);
				System.out.println("   "+ fmt.format(date.getTime()));
			}
		}
		/*for (Calendar c : analysisPeriod.holidays) {
			System.out.println("   "+ fmt.format(c.getTime()));
		}*/
		System.out.println("Dni w okresie                = " + analysisPeriod.getAllDaysPerPeriod());
		System.out.println("Dni pracujace w okresie      = " + analysisPeriod.getWorkDaysPerPeriod());
		System.out.println("Minuty dzien pracy           = " + minsPerDay);
		System.out.println("Minuty na okres              = " + totalMinsPerMonth);
		System.out.println("Minuty na zmiane             = " + minsPerWatch);
		System.out.println("Minimalna krotka zmiana(min) = " + minimumShortWatchMins);
		System.out.println("Limit zmian na tydzien       = " + weekWatchLimit);
		
		System.out.println("Limity personelu wedlug typu dnia:");
		for (Watch dayDef : daysDefinition.values()) {
			System.out.println("     " + dayDef);
		}
		
		for (int i = 0; i < employees.size(); i++) {
			Employee e = employees.get(i);
			System.out.println("Pracownik[" + (i + 1) + "] = " + e.getCode());
			
			//COPY-PASTE (TO REMOVE)!!!
			
			int workDays = analysisPeriod.getWorkDaysPerPeriod(employees.get(i).getLeaveOfAbsence());
			int minsPerPeriod = employeeMinsPerPeriod(i);
			//int minsPerPeriod = (int)(e.getMinsPerDay() * workDays);
			int watchesPerPeriod = minsPerPeriod / getMinsPerWatch();
			int remainingMins = minsPerPeriod % getMinsPerWatch();
			if ((remainingMins > 0) && (remainingMins < getMinimumShortWatchMins()) && (watchesPerPeriod > 0)) {
				watchesPerPeriod--;
				remainingMins = remainingMins + getMinsPerWatch();
			}
			int partWatchesPerPeriod = remainingMins > 0 ? (remainingMins / getMinsPerWatch()) + 1 : 0;
			
			System.out.println("   Dni pracujace w okresie["+(i + 1)+"]      = " + workDays);
			System.out.println("   Norma dobowa w minutach["+(i + 1)+"]      = " + e.getMinsPerDay());
			System.out.println("   Minuty pracy na okres["+(i + 1)+"]        = " + minsPerPeriod);
			System.out.println("   Pelne zmiany na okres["+(i + 1)+"]        = " + watchesPerPeriod);
			System.out.println("   Pozostale minuty pracy["+(i + 1)+"]       = " + remainingMins);
			System.out.println("   Liczba zmian krotkich["+(i + 1)+"]        = " + partWatchesPerPeriod);
			System.out.println("   Dni wolne w okresie["+(i + 1)+"]          = " + e.getLeaveOfAbsence().size());
			if (e.getNightsRequest() != null) {
				System.out.println("      nocki/dniowki = " + e.getNightsRequest() + "/" + (watchesPerPeriod - e.getNightsRequest()));
			}
			for (LeaveOfAbsence c : e.getLeaveOfAbsence()) {
				System.out.println("      urlop " + fmt.format(c.getDate().getTime()) + 
						" w wymiarze " + ((c.getAbsenceTime() == 0) ? "pelnym" : c.getAbsenceTime() + " minut"));
			}
			for (ScheduleRequest ar : e.getAbsenceRequest()) {
				System.out.println("      brak " + ar.getType() + " w " + fmt.format(ar.getDate().getTime()));
			}
			for (ScheduleRequest ar : e.getWorkRequest()) {
				System.out.println("      dyzur " + ar.getType() + " w " + fmt.format(ar.getDate().getTime()));
			}
		}
	}
}
