/*
  This file is part of Solver.

    Solver is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Solver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Solver. If not, see <http://www.gnu.org/licenses/>.
*/

package infra.definition;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import infra.Utils;

public class ProblemXMLLoader {
	public ProblemDefinition load(String fileName) {
		try {
			ProblemDefinition definition = new ProblemDefinition();
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(fileName);
			
			XPathFactory xPathfactory = XPathFactory.newInstance();
			XPath xpath = xPathfactory.newXPath();
			
			NodeList dayTypeNodes = (NodeList)xpath.compile("problem/dayTypes/dayType").evaluate(doc, XPathConstants.NODESET);
			for (int i = 0; i < dayTypeNodes.getLength(); i++) {
				String dayName = xpath.compile("./@name").evaluate(dayTypeNodes.item(i));
				String dayCode = xpath.compile("./@code").evaluate(dayTypeNodes.item(i));
				String minD = xpath.compile("./@minD").evaluate(dayTypeNodes.item(i));
				String maxD = xpath.compile("./@maxD").evaluate(dayTypeNodes.item(i));
				String minN = xpath.compile("./@minN").evaluate(dayTypeNodes.item(i));
				String maxN = xpath.compile("./@maxN").evaluate(dayTypeNodes.item(i));
				String minR = xpath.compile("./@minR").evaluate(dayTypeNodes.item(i));
				String maxR = xpath.compile("./@maxR").evaluate(dayTypeNodes.item(i));
				String minDR = xpath.compile("./@minDR").evaluate(dayTypeNodes.item(i));
				String maxDR = xpath.compile("./@maxDR").evaluate(dayTypeNodes.item(i));
				String min2DR = xpath.compile("./@min2DR").evaluate(dayTypeNodes.item(i));
				String max2DR = xpath.compile("./@max2DR").evaluate(dayTypeNodes.item(i));
				
				int minDV = Integer.parseInt(minD);
				int maxDV = Integer.parseInt(maxD);
				int minNV = Integer.parseInt(minN);
				int maxNV = Integer.parseInt(maxN);
				int minRV = Integer.parseInt(minR);
				int maxRV = Integer.parseInt(maxR);
				int minDRV = Utils.parseInt(minDR, -1);
				int maxDRV = Utils.parseInt(maxDR, -1);
				int min2DRV = Utils.parseInt(min2DR, -1);
				int max2DRV = Utils.parseInt(max2DR, -1);
				
				Watch w = new Watch(dayName, dayCode, minDV, maxDV, minNV, maxNV, minRV, maxRV, minDRV, maxDRV, min2DRV, max2DRV);
				definition.getDaysDefinition().put(dayCode, w);
			}
			
			Calendar startDate = parse(xpath.compile("problem/period/startDate/text()").evaluate(doc));
			Calendar endDate = parse(xpath.compile("problem/period/endDate/text()").evaluate(doc));
			
			String[] weekCodes = new String[7];
			NodeList weekNodes = (NodeList)xpath.compile("problem/period/weekDef/weekDay").evaluate(doc, XPathConstants.NODESET);
			for (int i = 0; i < weekNodes.getLength(); i++) {
				String day = xpath.compile("./@day").evaluate(weekNodes.item(i));
				String code = xpath.compile("./@code").evaluate(weekNodes.item(i));
				
				int dayValue = Integer.parseInt(day) - 1;
				if ((dayValue >= 0) && (dayValue < 7)) {
					weekCodes[dayValue] = code;
				}
			}
			
			//List<Calendar> holidays = new ArrayList<Calendar>();
			/*NodeList holidayNodes = (NodeList)xpath.compile("problem/period/days/holiday/text()").evaluate(doc, XPathConstants.NODESET);
			for (int i = 0; i < holidayNodes.getLength(); i++) {
				holidays.add(parse(holidayNodes.item(i).getNodeValue()));
			}*/
			
			Map<Calendar, String> days = new HashMap<Calendar, String>();
			NodeList dayNodes = (NodeList)xpath.compile("problem/period/dayDef/day").evaluate(doc, XPathConstants.NODESET);
			for (int i = 0; i < dayNodes.getLength(); i++) {
				String date = xpath.compile("./@date").evaluate(dayNodes.item(i));
				String code = xpath.compile("./@code").evaluate(dayNodes.item(i));
				days.put(parse(date), code);
				//String holiday = xpath.compile("./@holiday").evaluate(dayNodes.item(i));
			}
			
			TimePeriod period = new TimePeriod(startDate, endDate, weekCodes, days);
			definition.setAnalysisPeriod(period);
			
			NodeList employeeNodes = (NodeList)xpath.compile("problem/employees/employee").evaluate(doc, XPathConstants.NODESET);
			for (int i = 0; i < employeeNodes.getLength(); i++) {
				String employeeCode = xpath.compile("./@code").evaluate(employeeNodes.item(i));
				//System.out.println("CODE: " + employeeCode);
				
				List<LeaveOfAbsence> leaveOfAbsence = new ArrayList<LeaveOfAbsence>();
				NodeList loaDayNodes = (NodeList)xpath.compile("./days/leaveOfAbsence").evaluate(employeeNodes.item(i), XPathConstants.NODESET);
				for (int j = 0; j < loaDayNodes.getLength(); j++) {
					//Calendar date = parse(loaDayNodes.item(j).getNodeValue());
					String loaDate = xpath.compile("./text()").evaluate(loaDayNodes.item(j));
					String loaTime = xpath.compile("./@time").evaluate(loaDayNodes.item(j));
					Calendar date = parse(loaDate);
					int time = 0;
					if ((loaTime != null) && (!"".equals(loaTime))) {
						time = Integer.parseInt(loaTime);
					}
					leaveOfAbsence.add(new LeaveOfAbsence(date, time));
				}
				
				List<ScheduleRequest> absenceRequest = new ArrayList<ScheduleRequest>();
				NodeList absenceRequestNodes = (NodeList)xpath.compile("./days/absenceRequest").evaluate(employeeNodes.item(i), XPathConstants.NODESET);
				for (int j = 0; j < absenceRequestNodes.getLength(); j++) {
					String requestCode = xpath.compile("./@type").evaluate(absenceRequestNodes.item(j));
					String requestDate = xpath.compile("./text()").evaluate(absenceRequestNodes.item(j));
					absenceRequest.add(new ScheduleRequest(requestCode, parse(requestDate)));
				}
				
				List<ScheduleRequest> workRequest = new ArrayList<ScheduleRequest>();
				NodeList workRequestNodes = (NodeList)xpath.compile("./days/workRequest").evaluate(employeeNodes.item(i), XPathConstants.NODESET);
				for (int j = 0; j < workRequestNodes.getLength(); j++) {
					String requestCode = xpath.compile("./@type").evaluate(workRequestNodes.item(j));
					String requestDate = xpath.compile("./text()").evaluate(workRequestNodes.item(j));
					workRequest.add(new ScheduleRequest(requestCode, parse(requestDate)));
				}
				
				String dailyMinutes = xpath.compile("./@dailyMinutes").evaluate(employeeNodes.item(i));
				String nightsString = xpath.compile("./@nights").evaluate(employeeNodes.item(i));
				
				Integer nights = null;
				if ((nightsString != null) && (!nightsString.equals("")))
				{
					nights = Integer.parseInt(nightsString);
				}
				
				Employee employee = new Employee(employeeCode, leaveOfAbsence, absenceRequest, workRequest, nights);
				definition.getEmployees().add(employee);
				
				if ((dailyMinutes != null) && (!dailyMinutes.equals("")))
				{
					double value = Double.parseDouble(dailyMinutes);
					employee.setMinsPerDay(value);
				}
			}
			
			String weekLimitType = xpath.compile("problem/weekLimits/type/text()").evaluate(doc);
			if (weekLimitType != null) {
				if (weekLimitType.equals("calendar")) { definition.weeklyWorkload = ProblemDefinition.WeeklyWorkload.CALENDAR; }
				if (weekLimitType.equals("period")) { definition.weeklyWorkload = ProblemDefinition.WeeklyWorkload.PERIOD; }
				if (weekLimitType.equals("daily")) { definition.weeklyWorkload = ProblemDefinition.WeeklyWorkload.DAILY; }
			}
			
			
			//DEFAULT
			//int minsPerDay = 7 * 60 + 35;
			int minsPerWatch = 12 * 60;
			int minimumShortWatchMins = 4 * 60 + 30;
			int weekWatchLimit = 4;
						
			definition.setMinimumShortWatchMins(minimumShortWatchMins);
			//definition.setMinsPerDay(minsPerDay);
			definition.setMinsPerWatch(minsPerWatch);
			definition.setWeekWatchLimit(weekWatchLimit);
			
			//definition.getDaysDefinition().put("**", new Watch("**", 1, 1, 1, 1, 0, 0));
			//definition.getDaysDefinition().put("Z", new Watch("Z", 2, 3, 2, 2, 0, 1));
			
			return definition;
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}
	
	private Calendar parse(String text) {
		if (text == null) {
			return null;
		}
		DateFormat df = new SimpleDateFormat("yyyy.MM.dd");
		Date date = null;
		try {
			date = df.parse(text);
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		return cal;
	}
}
